#!/usr/bin/python

# fibonacci tutorial:
#
# 5 different ways to calculate the n'th fibonacci number:
# (0) - using someone else's code
# (1) - reading a text file with the f.n.'s
# (2) - recursion
# (3) - hashing + recursion
# (4) - lists
#
# call as (where N is some integer):
# $ python fibonacci_tutorial.py N
#
# lawrence david - 09.06.06

# note that we have to import some modules
import sys              # helpful for reading command line arguments
import fib_module       # contains code for calc. f.n.'s

# set the fibonacci number you want, using user input; note that you
# want an integer and command-line arguments are read as strings.  so,
# you need to cast the string as an int.

n = int(sys.argv[1])

#####
# 0 #
#####
# the simplest method of getting the fibonacci number: using someone
# else's code
fib0 = fib_module.calc_fib(n)

#####
# 1 #
#####
# query a datafile that has all the numbers stored within it
def fib1(n,filename):

    # open a file for reading
    f = open(filename,'r')

    # dump each line into a list
    L = f.readlines()

    # use a while loop to step through each line of the text file
    counter = 0
    while counter < n:

        # strip pulls newlines off of the ends of strings
        fibnum = L[counter].strip()
        counter += 1

    # be tidy!
    f.close()

    return fibnum 

#####
# 2 #
#####
# vanilla recursion
def fib2(n):

    # stop conditions
    if n == 1:
        return 1
    elif n == 2:
        return 2

    # recursive step
    else:
        return fib2(n-1)+fib2(n-2)

#####
# 3 #
#####
# using hashing to speed up recursion.  note the second argument: it
# says that if a second argument is not defined, pass an empty dict
# (hash table)
def fib3(n,ht={}):

    # stop condition - query hash table to see if we've already
    # calculated this fibonacci number
    
    if ht.has_key(n):
        return ht[n]

    # if this is a new number, default to our earlier strategy of
    # calculating fibonacci numbers.  note that you can condense the
    # previous if, elif sequence using an 'or'
    else:
        if n == 1 or n == 2:
            ht[n] = n
        else:
            ht[n] = fib3(n-1,ht)+fib3(n-2,ht)
        return ht[n]

#####
# 4 #
#####
# use lists to calculate the fibonacci number
def fib4(n):

    # initiate a list
    seq_list = []

    # use a for loop, that iterates from 1 to n+1
    for i in range(1,n+1):
        if i == 1:
            seq_list.append(1)
        elif i == 2:
            seq_list.append(2)

        # get past 2 numbers from the list
        else:
            seq_list.append(seq_list[i-2] + seq_list[i-3])

    # note that lists can be indexed 'backwards'; -1 is the same as
    # the n'th (last) index
    return seq_list[-1]

# run each method:
print "\nhere's the " + str(n) + "th fibonacci number:\n"
print "method 0:", 
print fib0
print "method 1:",
print fib1(n,"fib.txt")
print "method 2:",
print fib2(n)
print "method 3:",
print fib3(n)
print "method 4:",
print fib4(n)




